/* ple.c */

#ifdef __APPLE__
#include <OpenGL/gl.h>
#else
#include <GL/glew.h>
#include <GL/gl.h>
#endif

#include <math.h>
#include <stdlib.h>

#include "ple.h"


#define	X(p)		((p)->x)
#define	Y(p)		((p)->y)
#define	c_new(t)	((t*)malloc(sizeof(t)))
#define	next(p)		((p)->next)
#define	prev(p)		((p)->prev)

#define TOL 10

static Point*		firstpoint=NULL;
static Point*		lastpoint=NULL;
static int          showingpolygon=1;

Point* moving=NULL;

#define BG_COL 0.85
#define FG_COL 1.0

void redraw(int clear)
{
    if (clear)
        dvclear(BG_COL);
    if (showingpolygon)
        showpolygon();
    showpoints();
}

void delpoints(void)
{
    firstpoint=lastpoint=NULL;			/* lazy! */
}

void addpoint(real x, real y)
{
    Point* p = c_new(Point);
    X(p)=x;
    Y(p)=y;
    next(p)=NULL;
    if (firstpoint==NULL) {
        prev(p)=NULL;
        firstpoint=p;
    } else {
        prev(p)=lastpoint;
        next(lastpoint)=p;
    }
    lastpoint=p;
}

void delpoint(real x, real y)
{
    Point* p=findpoint(x,y);
    if (p!=NULL) {
        if (prev(p)==NULL) firstpoint=next(p); else next(prev(p))=next(p);
        if (next(p)==NULL) lastpoint=prev(p);  else prev(next(p))=prev(p);
    }
}

void startmove(real x, real y)
{
    moving = findpoint(x,y);
    if (moving!=NULL) {
        x=X(moving); y=Y(moving);
    }
}

void movepoint(real x, real y)
{
    if (moving!=NULL) {
        X(moving)=x; Y(moving)=y;
    } else {
        startmove(x,y);
    }
}

void endmove(void)
{
    if (moving!=NULL) {
        moving=NULL;
    }
}

Point* findpoint(real x, real y)
{
    Point* p=firstpoint;
    for (p=firstpoint; p!=NULL; p=next(p)) {
        if ((fabs(X(p)-x)+fabs(Y(p)-y))<TOL)
            break;
    }
    return p;
}

void showpoints(void)
{
    Point* p;
    for (p=firstpoint; p!=NULL; p=next(p))
        showpoint(p);
}

void showpolygon(void)
{
    Point* p;
    for (p=firstpoint; p!=NULL; p=next(p))
        showside(p,next(p));
}

void showpoint(Point* p)
{
    dvplot(X(p),Y(p));
}

void showside(Point* p, Point *q)
{
    if (p!=NULL && q!=NULL)
        dvline(X(p),Y(p),X(q),Y(q));
}

int is_moving(void)
{
    return (moving != NULL);
}

void toggleshowpolygon(void)
{
    showingpolygon = !showingpolygon;
}


// -----------------------------------------


static double dv_w, dv_h;

void dvflush(void)
{
    glFlush();
}

void dvclear(double c)
{
    glClearColor(c, c, c, 0.f);
    glClear(GL_COLOR_BUFFER_BIT);
}

void dvcolor(double c)
{
    glColor3d(c, c, c);
}

void dvwindow(double xmax, double ymax)
{
    dv_w = xmax;
    dv_h = ymax;
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    glOrtho((GLdouble)(0), (GLdouble)(xmax), (GLdouble)(0), (GLdouble)(ymax), -1.0, 1.0);
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity();
}

void dvplot(double x1, double y1)
{
    double s = 5.0;
    glRectd((GLdouble)(x1-s), (GLdouble)(y1-s), (GLdouble)(x1+s), (GLdouble)(y1+s));
}

void dvline(double x1, double y1, double x2, double y2)
{
    glBegin(GL_LINES);
    glVertex2d((GLdouble)(x1), (GLdouble)(y1));
    glVertex2d((GLdouble)(x2), (GLdouble)(y2));
    glEnd();
}

real dv_map_y(real y)
{
    return dv_h - y;
}
